@@ -31,6 +31,12 @@ RT_ROUTER = 'router'
 RT_NS_ROUTER = 'ns_router'
 RT_SG = 'security_group'
 
+REAL_SHADOW_TYPE_MAP = {
+    RT_NETWORK: RT_SD_NETWORK,
+    RT_SUBNET: RT_SD_SUBNET,
+    RT_PORT: RT_SD_PORT
+}
+
 
 # check whether the resource type is properly provisioned.
 def is_valid_resource_type(resource_type):
@@ -75,6 +81,12 @@ SP_EXTRA_ID = '00000000-0000-0000-0000-000000000000'
 TOP = 'top'
 POD_NOT_SPECIFIED = 'not_specified_pod'
 PROFILE_REGION = 'region'
+PROFILE_DEVICE = 'device'
+PROFILE_HOST = 'host'
+PROFILE_AGENT_TYPE = 'type'
+PROFILE_TUNNEL_IP = 'tunnel_ip'
+PROFILE_FORCE_UP = 'force_up'
+DEVICE_OWNER_SHADOW = 'compute:shadow'
 
 # job type
 JT_ROUTER = 'router'
@@ -83,6 +95,14 @@ JT_PORT_DELETE = 'port_delete'
 JT_SEG_RULE_SETUP = 'seg_rule_setup'
 JT_NETWORK_UPDATE = 'update_network'
 JT_SUBNET_UPDATE = 'subnet_update'
+JT_SHADOW_PORT_SETUP = 'shadow_port_setup'
+
 # network type
 NT_LOCAL = 'local'
 NT_VLAN = 'vlan'
+NT_VxLAN = 'vxlan'
+
+# cross-pod VxLAN networking support mode
+NM_P2P = 'p2p'
+NM_L2GW = 'l2gw'
+NM_NOOP = 'noop'
