@@ -242,12 +242,21 @@ def get_bottom_mappings_by_tenant_pod(context,
     return routings
 
 
-def delete_mappings_by_top_id(context, top_id):
+def delete_mappings_by_top_id(context, top_id, pod_id=None):
+    """Delete resource routing entry based on top resource ID
+
+    If pod ID is also provided, only entry in the specific pod will be deleted
+
+    :param context: context object
+    :param top_id: top resource ID
+    :param pod_id: optional pod ID
+    :return: None
+    """
+    filters = [{'key': 'top_id', 'comparator': 'eq', 'value': top_id}]
+    if pod_id:
+        filters.append({'key': 'pod_id', 'comparator': 'eq', 'value': pod_id})
     with context.session.begin():
-        core.delete_resources(
-            context, models.ResourceRouting,
-            filters=[{'key': 'top_id', 'comparator': 'eq',
-                      'value': top_id}])
+        core.delete_resources(context, models.ResourceRouting, filters=filters)
 
 
 def delete_mappings_by_bottom_id(context, bottom_id):
@@ -453,6 +462,37 @@ def finish_job(context, job_id, successful, timestamp):
                 synchronize_session=False)
 
 
+def ensure_agent_exists(context, pod_id, host, _type, tunnel_ip):
+    try:
+        context.session.begin()
+        agents = core.query_resource(
+            context, models.ShadowAgent,
+            [{'key': 'host', 'comparator': 'eq', 'value': host},
+             {'key': 'type', 'comparator': 'eq', 'value': _type}], [])
+        if agents:
+            return
+        core.create_resource(context, models.ShadowAgent,
+                             {'id': uuidutils.generate_uuid(),
+                              'pod_id': pod_id,
+                              'host': host,
+                              'type': _type,
+                              'tunnel_ip': tunnel_ip})
+        context.session.commit()
+    except db_exc.DBDuplicateEntry:
+        # agent has already been created
+        context.session.rollback()
+    finally:
+        context.session.close()
+
+
+def get_agent_by_host_type(context, host, _type):
+    agents = core.query_resource(
+        context, models.ShadowAgent,
+        [{'key': 'host', 'comparator': 'eq', 'value': host},
+         {'key': 'type', 'comparator': 'eq', 'value': _type}], [])
+    return agents[0] if agents else None
+
+
 def _is_user_context(context):
     """Indicates if the request context is a normal user."""
     if not context:
