@@ -27,4 +27,14 @@ class DefaultGroupUpdateNotSupported(exceptions.InvalidInput):
 
 
 class BottomPodOperationFailure(exceptions.NeutronException):
-    message = _('Operation for %(resource)s on bottom pod %(pod_name)s fails')
+    message = _(
+        'Operation for %(resource)s on bottom pod %(region_name)s fails')
+
+
+class DhcpPortNotFound(exceptions.NotFound):
+    message = _('Dhcp port for subnet %(subnet_id)s not found')
+
+
+class GatewayPortNotFound(exceptions.NotFound):
+    message = _('Gateway port for subnet %(subnet_id)s and region %(region)s '
+                'not found')
